# comanage_api/_copersonroles.py

"""
CoPersonRole API - https://spaces.at.internet2.edu/display/COmanage/CoPersonRole+API

Methods
-------
copersonroles_add(coperson_id: int, cou_id: int, status: str = None, affiliation: str = None) -> dict
    Add a new CO Person Role.
copersonroles_delete(copersonrole_id: int) -> bool
    Remove a CO Person Role.
copersonroles_edit(copersonrole_id: int, coperson_id: int = None, cou_id: int = None, status: str = None,
                   affiliation: str = None) -> bool
    Edit an existing CO Person Role.
copersonroles_view_all() -> dict
    Retrieve all existing CO Person Roles.
copersonroles_view_per_coperson(coperson_id: int) -> dict
    Retrieve all existing CO Person Roles for the specified CO Person. Available since Registry v2.0.0.
copersonroles_view_per_cou(cou_id: int) -> dict
    Retrieve all existing CO Person Roles for the specified COU.
copersonroles_view_one(copersonrole_id: int) -> dict
    Retrieve an existing CO Person Role.
"""

import json


def copersonroles_add(self, coperson_id: int, cou_id: int, status: str = None, affiliation: str = None) -> dict:
    """
    Add a new CO Person Role.

    :param self:
    :param affiliation:
    :param coperson_id:
    :param cou_id:
    :param status:

    :return
        {
            "ResponseType":"NewObject",
            "Version":"1.0",
            "ObjectType":"CoPersonRole",
            "Id":"<ID>"
        }:

    Request Format
        {
          "RequestType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Person":
              {
                "Type":"CO",
                "Id":"<coperson_id>"
              },
              "CouId":"<cou_id>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            }
          ]
        }

    Response Format
        HTTP Status             Response Body                       Description
        201 Added               NewObjectResponse with ObjectType   CoPersonRole created
        400 Bad Request                                             CoPersonRole Request not provided in POST body
        400 Invalid Fields      ErrorResponse with details in       An error in one or more provided fields
                                InvalidFields element
        401 Unauthorized                                            Authentication required
        403 COU Does Not Exist                                      The specified COU does not exist
        500 Other Error                                             Unknown error
    """
    post_body = {
        'RequestType': 'CoPersonRoles',
        'Version': '1.0',
        'CoPersonRoles': [
            {
                'Version': '1.0',
                'Person':
                    {
                        'Type': 'CO',
                        'Id': str(coperson_id)
                    },
                'CouId': str(cou_id),
                'O': str(self._CO_API_ORG_NAME)
            }
        ]
    }
    if status:
        if status not in self.STATUS_OPTIONS:
            raise TypeError("Invalid Fields 'status'")
        post_body['CoPersonRoles'][0]['Status'] = str(status)
    else:
        post_body['CoPersonRoles'][0]['Status'] = 'Active'
    if affiliation:
        affiliation = str(affiliation).lower()
        if affiliation not in self.AFFILIATION_OPTIONS:
            raise TypeError("Invalid Fields 'affiliation'")
        post_body['CoPersonRoles'][0]['Affiliation'] = str(affiliation)
    else:
        post_body['CoPersonRoles'][0]['Affiliation'] = 'member'
    post_body = json.dumps(post_body)
    url = self._CO_API_URL + '/co_person_roles.json'
    resp = self._s.post(
        url=url,
        data=post_body
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copersonroles_delete(self, copersonrole_id: int) -> bool:
    """
    Remove a CO Person Role.

    :param self:
    :param copersonrole_id:
    :return:

    Response Format
        HTTP Status                 Response Body       Description
        200 Deleted                                     CoPersonRole deleted
        400 Invalid Fields                              id not provided
        401 Unauthorized                                Authentication required
        404 CoPersonRole Unknown                        id not found
        500 Other Error                                 Unknown error
    """
    url = self._CO_API_URL + '/co_person_roles/' + str(copersonrole_id) + '.json'
    resp = self._s.delete(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def copersonroles_edit(self, copersonrole_id: int, coperson_id: int = None, cou_id: int = None, status: str = None,
                       affiliation: str = None) -> bool:
    """
    Edit an existing CO Person Role.

    :param self:
    :param copersonrole_id:
    :param affiliation:
    :param coperson_id:
    :param cou_id:
    :param status:

    :return:

    Request Format
        {
          "RequestType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Person":
              {
                "Type":"CO",
                "Id":"<coperson_id>"
              },
              "CouId":"<cou_id>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            }
          ]
        }

    Response Format
        HTTP Status                 Response Body                       Description
        200 OK                                                          CoPersonRole updated
        400 Bad Request                                                 CoPersonRole Request not provided in POST body
        400 Invalid Fields          ErrorRespons with details in        An error in one or more provided fields
                                    InvalidFields element
        401 Unauthorized                                                Authentication required
        403 COU Does Not Exist                                          The specified COU does not exist
        404 CoPersonRole Unknown                                        id not found
        500 Other Error                                                 Unknown error
    """
    copersonrole = copersonroles_view_one(self, copersonrole_id)
    post_body = {
        'RequestType': 'CoPersonRoles',
        'Version': '1.0',
        'CoPersonRoles': [
            {
                'Version': '1.0',
                'Person':
                    {
                        'Type': 'CO'
                    },
                'O': str(self._CO_API_ORG_NAME)
            }
        ]
    }
    if coperson_id:
        post_body['CoPersonRoles'][0]['Person']['Id'] = str(coperson_id)
    else:
        post_body['CoPersonRoles'][0]['Person']['Id'] = str(
            copersonrole.get('CoPersonRoles')[0].get('Person').get('Id'))
    if cou_id:
        post_body['CoPersonRoles'][0]['CouId'] = str(cou_id)
    else:
        post_body['CoPersonRoles'][0]['CouId'] = str(copersonrole.get('CoPersonRoles')[0].get('CouId'))
    if status:
        if status not in self.STATUS_OPTIONS:
            raise TypeError("Invalid Fields 'status'")
        post_body['CoPersonRoles'][0]['Status'] = str(status)
    else:
        post_body['CoPersonRoles'][0]['Status'] = copersonrole.get('CoPersonRoles')[0].get('Status')
    if affiliation:
        affiliation = str(affiliation).lower()
        if affiliation not in self.AFFILIATION_OPTIONS:
            raise TypeError("Invalid Fields 'affiliation'")
        post_body['CoPersonRoles'][0]['Affiliation'] = str(affiliation)
    else:
        post_body['CoPersonRoles'][0]['Affiliation'] = copersonrole.get('CoPersonRoles')[0].get('Affiliation')
    post_body = json.dumps(post_body)
    url = self._CO_API_URL + '/co_person_roles/' + str(copersonrole_id) + '.json'
    resp = self._s.put(
        url=url,
        data=post_body
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def copersonroles_view_all(self) -> dict:
    """
    Retrieve all existing CO Person Roles.

    :param self:
    :return
        {
          "ResponseType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Id":"<Id>",
              "Person":
              {
                "Type":"CO",
                "Id":"<ID>"
              },
              "CouId":"<CouId>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|"GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status             Response Body                       Description
        200 OK                  CoPersonRole Response               CoPersonRoles returned
        401 Unauthorized                                            Authentication required
        500 Other Error                                             Unknown error
    """
    url = self._CO_API_URL + '/co_person_roles.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copersonroles_view_per_coperson(self, coperson_id: int) -> dict:
    """
    Retrieve all existing CO Person Roles for the specified CO Person. Available since Registry v2.0.0.

    :param self:
    :param coperson_id:
    :return
        {
          "ResponseType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Id":"<Id>",
              "Person":
              {
                "Type":"CO",
                "Id":"<ID>"
              },
              "CouId":"<CouId>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|"GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status             Response Body                       Description
        200 OK                  CoPersonRole Response               CoPersonRoles returned
        401 Unauthorized                                            Authentication required
        404 CO Person Unknown                                       id not found
        500 Other Error                                             Unknown error
    """
    url = self._CO_API_URL + '/co_person_roles.json'
    params = {'copersonid': int(coperson_id)}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copersonroles_view_per_cou(self, cou_id: int) -> dict:
    """
    Retrieve all existing CO Person Roles for the specified COU.

    :param self:
    :param cou_id:
    :return
        {
          "ResponseType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Id":"<Id>",
              "Person":
              {
                "Type":"CO",
                "Id":"<ID>"
              },
              "CouId":"<CouId>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|"GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status             Response Body                       Description
        200 OK                  CoPersonRole Response               CoPersonRoles returned
        401 Unauthorized                                            Authentication required
        404 COU Unknown                                             id not found
        500 Other Error                                             Unknown error
    """
    url = self._CO_API_URL + '/co_person_roles.json'
    params = {'couid': int(cou_id)}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copersonroles_view_one(self, copersonrole_id: int) -> dict:
    """
    Retrieve an existing CO Person Role.

    :param self:
    :param copersonrole_id:
    :return
        {
          "ResponseType":"CoPersonRoles",
          "Version":"1.0",
          "CoPersonRoles":
          [
            {
              "Version":"1.0",
              "Id":"<Id>",
              "Person":
              {
                "Type":"CO",
                "Id":"<ID>"
              },
              "CouId":"<CouId>",
              "Affiliation":"<Affiliation>",
              "Title":"<Title>",
              "O":"<O>",
              "Ordr":"<Order>",
              "Ou":"<Ou>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|"GracePeriod"|"Invited"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended"),
              "ValidFrom":"<ValidFrom>",
              "ValidThrough":"<ValidThrough>",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>",
              "ExtendedAttributes":
              {
                "<Attribute>":"<Value>",
                {...}
              }
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status                 Response Body                       Description
        200 OK                      CoPersonRole Response               CoPersonRoles returned
        401 Unauthorized                                                Authentication required
        404 CoPersonRole Unknown                                        id not found
        500 Other Error                                                 Unknown error
    """
    url = self._CO_API_URL + '/co_person_roles/' + str(copersonrole_id) + '.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



# comanage_api/_cous.py

"""
COU API - https://spaces.at.internet2.edu/display/COmanage/COU+API

Methods
-------
cous_add(name: str, description: str, parent_id: int = None) -> dict
    Add a new Cou.
cous_delete(cou_id: int) -> bool
    Remove a Cou.
cous_edit(cou_id: int, name: str = None, description: str = None, parent_id: int = None) -> bool
    Edit an existing Cou.
cous_view_all() -> dict
    Retrieve all existing Cous.
cous_view_per_co() -> dict
    Retrieve Cou attached to a CO.
cous_view_one(cou_id: int) -> dict
    Retrieve an existing Cou.
"""

import json


def cous_add(self, name: str, description: str, parent_id: int = None) -> dict:
    """
    Add a new Cou.

    :param self:
    :param name:
    :param description:
    :param parent_id:
    :return
        {
            "ResponseType":"NewObject",
            "Version":"1.0",
            "ObjectType":"Cou",
            "Id":"<INTEGER>"
        }:

    Request Format
        {
          "RequestType":"Cous",
          "Version":"1.0",
          "Cous":
          [
            {
              "Version":"1.0",
              "CoId":"<CO_API_ORG_ID>",
              "ParentId":"<parent_id>",
              "Name":"<name>",
              "Description":"<description>",
            }
          ]
        }

    Response Format
        HTTP Status             Response Body                       Description
        201 Added               NewObjectResponse with ObjectType   Cou added
        400 Bad Request                                             Cou Request not provided in POST body
        400 Invalid Fields      ErrorRespons with details in        An error in one or more provided fields
                                InvalidFields element
        401 Unauthorized                                            Authentication required
        403 CO Does Not Exist                                       The specified CO does not exist
        403 Name In Use                                             A COU already exists with the specified name in
                                                                    the specified CO
        403 Parent Would                                            Parent COU can not be a descendant of the child
        Create Cycle
        403 Wrong CO                                                Parent/Child COU not member of same CO
        500 Other Error                                             Unknown error
    """
    post_body = {
        'RequestType': 'Cous',
        'Version': '1.0',
        'Cous':
            [
                {
                    'Version': '1.0',
                    'CoId': self._CO_API_ORG_ID,
                    'Name': str(name),
                    'Description': str(description)
                }
            ]
    }
    if parent_id:
        post_body['Cous'][0]['ParentId'] = str(parent_id)
    post_body = json.dumps(post_body)
    url = self._CO_API_URL + '/cous.json'
    resp = self._s.post(
        url=url,
        data=post_body
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def cous_delete(self, cou_id: int) -> bool:
    """
    Remove a Cou.

    :param self:
    :param cou_id:
    :return:

    Response Format
        HTTP Status                 Response Body       Description
        200 Deleted                                     Cou deleted
        400 Invalid Fields                              id not provided
        401 Unauthorized                                Authentication required
        403 CoPersonRole Exists                         One or more CO Person Roles are members of this COU,
                                                        and so the COU cannot be removed
        404 Identifier Unknown                          id not found
        500 Other Error                                 Unknown error
    """
    url = self._CO_API_URL + '/cous/' + str(cou_id) + '.json'
    params = {'coid': self._CO_API_ORG_ID}
    resp = self._s.delete(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def cous_edit(self, cou_id: int, name: str = None, description: str = None, parent_id: int = None) -> bool:
    """
    Edit an existing Cou.

    :param self:
    :param cou_id:
    :param name:
    :param description:
    :param parent_id:
    :return
        {
            "status_code": 200,
            "reason": "OK"
        }:

    Request Format
        {
          "RequestType":"Cous",
          "Version":"1.0",
          "Cous":
          [
            {
              "Version":"1.0",
              "CoId":"<CO_API_ORG_ID>",
              "ParentId":"<parent_id>",
              "Name":"<name>",
              "Description":"<description>",
            }
          ]
        }

    Response Format
        HTTP Status             Response Body                       Description
        200 OK                                                      Cou updated
        400 Bad Request                                             Cou Request not provided in POST body
        400 Invalid Fields      ErrorRespons with details in        An error in one or more provided fields
                                InvalidFields element
        401 Unauthorized                                            Authentication required
        403 CO Does Not Exist                                       The specified CO does not exist
        403 Name In Use                                             A COU already exists with the specified name in
                                                                    the specified CO
        403 Parent Would                                            Parent COU can not be a descendant of the child
        Create Cycle
        403 Wrong CO                                                Parent/Child COU not member of same CO
        404 Identifier Unknown                                      id not found
        500 Other Error                                             Unknown error
    """
    cou = cous_view_one(self, cou_id)
    post_body = {
        'RequestType': 'Cous',
        'Version': '1.0',
        'Cous':
            [
                {
                    'Version': '1.0',
                    'CoId': self._CO_API_ORG_ID
                }
            ]
    }
    if name:
        post_body['Cous'][0]['Name'] = str(name)
    else:
        post_body['Cous'][0]['Name'] = cou.get('Cous')[0].get('Name')
    if description:
        post_body['Cous'][0]['Description'] = str(description)
    else:
        post_body['Cous'][0]['Description'] = cou.get('Cous')[0].get('Description')
    if parent_id:
        post_body['Cous'][0]['ParentId'] = str(parent_id)
    else:
        if cou.get('Cous')[0].get('ParentId'):
            post_body['Cous'][0]['ParentId'] = str(cou.get('Cous')[0].get('ParentId'))
        if str(parent_id) == '0':
            post_body['Cous'][0]['ParentId'] = ''
    post_body = json.dumps(post_body)
    url = self._CO_API_URL + '/cous/' + str(cou_id) + '.json'
    resp = self._s.put(
        url=url,
        data=post_body
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def cous_view_all(self) -> dict:
    """
    Retrieve all existing Cous.

    :param self:
    :return
    {
        "ResponseType":"Cous",
        "Version":"1.0",
        "Cous":[
            {
                "Version":"1.0",
                "Id":"<INTEGER>",
                "CoId":"<CO_API_ORG_ID>",
                "Name":"<name>",
                "Description":"<description>",
                "Lft":"64",
                "Rght":"65",
                "Created":"2021-09-14 14:53:02",
                "Modified":"2021-09-14 14:53:02",
                "Revision":"0",
                "Deleted":false,
                "ActorIdentifier":"<COmanage_ID>"
            },
            {
                ...
            }
        ]
    }:

    Response Format
        HTTP Status         Response Body       Description
        200 OK              Cou Response        Cou returned
        401 Unauthorized                        Authentication required
        500 Other Error                         Unknown error
    """
    url = self._CO_API_URL + '/cous.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def cous_view_per_co(self) -> dict:
    """
    Retrieve Cou attached to a CO.

    :param self:
    :return
    {
        "ResponseType":"Cous",
        "Version":"1.0",
        "Cous":[
            {
                "Version":"1.0",
                "Id":"<INTEGER>",
                "CoId":"<CO_API_ORG_ID>",
                "Name":"<name>",
                "Description":"<description>",
                "Lft":"64",
                "Rght":"65",
                "Created":"2021-09-14 14:53:02",
                "Modified":"2021-09-14 14:53:02",
                "Revision":"0",
                "Deleted":false,
                "ActorIdentifier":"<COmanage_ID>"
            },
            {
                ...
            }
        ]
    }:

    Response Format
        HTTP Status         Response Body       Description
        200 OK              Cou Response        Cou returned
        401 Unauthorized                        Authentication required
        404 CO Unknown                          id not found
        500 Other Error                         Unknown error
    """
    url = self._CO_API_URL + '/cous.json'
    params = {'coid': self._CO_API_ORG_ID}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def cous_view_one(self, cou_id: int) -> dict:
    """
    Retrieve an existing Cou.

    :param self:
    :param cou_id:
    :return
    {
        "ResponseType":"Cous",
        "Version":"1.0",
        "Cous":[
            {
                "Version":"1.0",
                "Id":"<INTEGER>",
                "CoId":"<CO_API_ORG_ID>",
                "Name":"<name>",
                "Description":"<description>",
                "Lft":"64",
                "Rght":"65",
                "Created":"2021-09-14 14:53:02",
                "Modified":"2021-09-14 14:53:02",
                "Revision":"0",
                "Deleted":false,
                "ActorIdentifier":"<COmanage_ID>"
            }
        ]
    }:

    Response Format
        HTTP Status         Response Body       Description
        200 OK              Cou Response        Cou returned
        401 Unauthorized                        Authentication required
        404 COU Unknown                         id not found
        500 Other Error                         Unknown error
    """
    url = self._CO_API_URL + '/cous/' + str(cou_id) + '.json'
    params = {'coid': self._CO_API_ORG_ID}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



# comanage_api/_sshkeys.py

"""
SshKey API - https://spaces.at.internet2.edu/display/COmanage/SshKey+API

Methods
-------
ssh_keys_add(coperson_id: int, ssh_key: str, key_type: str, comment: str = None,
             ssh_key_authenticator_id: int = None) -> dict
    Add a new SSH Key.
ssh_keys_delete(ssh_key_id: int) -> bool
    Remove an SSH Key.
ssh_keys_edit(ssh_key_id: int, coperson_id: int = None, ssh_key: str = None, key_type: str = None,
              comment: str = None, ssh_key_authenticator_id: int = None) -> bool
    Edit an exiting SSH Key.
ssh_keys_view_all() -> dict
    Retrieve all existing SSH Keys.
ssh_keys_view_per_coperson(coperson_id: int) -> dict
    Retrieve all existing SSH Keys for the specified CO Person.
ssh_keys_view_one(ssh_key_id: int) -> dict
    Retrieve an existing SSH Key.

Notes
-----
Experimental
- The SshKey API is implemented via the SSH Key Authenticator Plugin.
  REST APIs provided by plugins are currently considered Experimental, and as such this interface may change
  without notice between minor releases.

Implementation Notes
- Only JSON format is supported. XML format is not supported.
- Note the URLs for this API use plugin syntax. (There is an extra component to the path.)
- As defined in the SshKey Schema, an SSH Key Authenticator ID is required as part of the request.
  This refers to the Authenticator instantiated for the CO.
- Authenticators that are locked cannot be managed by the API.
"""

import json


def ssh_keys_add(self, coperson_id: int, ssh_key: str, key_type: str, comment: str = None,
                 ssh_key_authenticator_id: int = None) -> dict:
    """
    Add a new SSH Key.

    :param self:
    :param coperson_id:
    :param ssh_key:
    :param key_type:
    :param comment:
    :param ssh_key_authenticator_id:
    :return
        {
            "ResponseType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                    "Version":"1.0",
                    "Id":"<Id>",
                    "Person":
                    {
                        "Type":"CO",
                        "Id":"<ID>"
                    },
                    ,
                    "Comment":"<Comment>",
                    "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                            "ssh-ed25519'"|"ssh-rsa"|"ssh-rsa1"),
                    "Skey":"<SshKey>",
                    "SshKeyAuthenticatorId":"<Id>"
                },
                {...}
            ]
        }:

    Request Format
        {
            "RequestType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                    "Version":"1.0",
                    "Person":
                    {
                        "Type":"CO",
                        "Id":"<ID>"
                    },
                    "Comment":"<Comment>",
                    "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                            "ssh-ed25519'"|"ssh-rsa"|"ssh-rsa1"),
                    "Skey":"<SshKey>",
                    "SshKeyAuthenticatorId":"<Id>"
                }
            ]
        }:

    Response Format
        HTTP Status             Response Body                       Description
        201 Added               NewObjectResponse with              SSH Key created
                                ObjectType of SshKey
        400 Bad Request                                             SSH Key Request not provided in POST body
        400 Invalid Fields      ErrorResponse with details in       An error in one or more provided fields
                                InvalidFields element
        401 Unauthorized                                            Authentication required
        500 Other Error                                             Unknown error
    """
    if not ssh_key_authenticator_id:
        ssh_key_authenticator_id = self._CO_SSH_KEY_AUTHENTICATOR_ID
    key_type = str(key_type).lower()
    if key_type not in self.SSH_KEY_OPTIONS:
        raise TypeError("Invalid Fields 'key_type'")
    post_body = json.dumps({
        'RequestType': 'SshKeys',
        'Version': '1.0',
        'SshKeys':
            [
                {
                    'Version': '1.0',
                    'Person':
                        {
                            'Type': 'CO',
                            'Id': str(coperson_id)
                        },
                    'Comment': str(comment),
                    'Type': str(key_type),
                    'Skey': str(ssh_key),
                    'SshKeyAuthenticatorId': str(ssh_key_authenticator_id)
                }
            ]
    })
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys.json'
    resp = self._s.post(
        url=url,
        data=post_body
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def ssh_keys_delete(self, ssh_key_id: int) -> bool:
    """
    Remove an SSH Key.

    :param self:
    :param ssh_key_id:
    :return::

    Response Format
        HTTP Status             Response Body       Description
        200 OK                                      SSH Key updated
        400 Invalid Fields                          An error in one or more provided fields
        401 Unauthorized                            Authentication required
        404 SshKey Unknown                          id not found
        500 Other Error                             Unknown error
    """
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys/' + str(ssh_key_id) + '.json'
    resp = self._s.delete(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def ssh_keys_edit(self, ssh_key_id: int, coperson_id: int = None, ssh_key: str = None, key_type: str = None,
                  comment: str = None, ssh_key_authenticator_id: int = None) -> bool:
    """
    Edit an exiting SSH Key.

    :param self:
    :param ssh_key_id:
    :param coperson_id:
    :param ssh_key:
    :param key_type:
    :param comment:
    :param ssh_key_authenticator_id:
    :return:

    Request Format
        {
            "RequestType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                    "Version":"1.0",
                    "Person":
                    {
                        "Type":"CO",
                        "Id":"<ID>"
                    },
                    "Comment":"<Comment>",
                    "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                            "ssh-ed25519'"|"ssh-rsa"|"ssh-rsa1"),
                    "Skey":"<SshKey>",
                    "SshKeyAuthenticatorId":"<Id>"
                }
            ]
        }:

    Response Format
        HTTP Status             Response Body                       Description
        200 OK                                                      SSH Key updated
        400 Bad Request                                             SSH Key Request not provided in POST body
        400 Invalid Fields      ErrorResponse with details in       An error in one or more provided fields
                                InvalidFields element
        401 Unauthorized                                            Authentication required
        404 SshKey Unknown                                          id not found
        500 Other Error                                             Unknown error
    """
    sshkey = ssh_keys_view_one(self, ssh_key_id=ssh_key_id)
    post_body = {
        'RequestType': 'SshKeys',
        'Version': '1.0',
        'SshKeys':
            [
                {
                    'Version': '1.0',
                    'Person':
                        {
                            'Type': 'CO'
                        }
                }
            ]
    }
    if coperson_id:
        post_body['SshKeys'][0]['Person']['Id'] = str(coperson_id)
    else:
        post_body['SshKeys'][0]['Person']['Id'] = str(sshkey.get('SshKeys')[0].get('Person').get('Id'))
    if ssh_key:
        post_body['SshKeys'][0]['Skey'] = str(ssh_key)
    else:
        post_body['SshKeys'][0]['Skey'] = sshkey.get('SshKeys')[0].get('Skey')
    if key_type:
        key_type = str(key_type).lower()
        if key_type not in self.SSH_KEY_OPTIONS:
            raise TypeError("Invalid Fields 'key_type'")
        post_body['SshKeys'][0]['Type'] = str(key_type)
    else:
        post_body['SshKeys'][0]['Type'] = sshkey.get('SshKeys')[0].get('Type')
    if comment:
        post_body['SshKeys'][0]['Comment'] = str(comment)
    else:
        post_body['SshKeys'][0]['Comment'] = sshkey.get('SshKeys')[0].get('Comment')
    if ssh_key_authenticator_id:
        post_body['SshKeys'][0]['SshKeyAuthenticatorId'] = str(ssh_key_authenticator_id)
    else:
        post_body['SshKeys'][0]['SshKeyAuthenticatorId'] = str(sshkey.get('SshKeys')[0].get('SshKeyAuthenticatorId'))
    post_body = json.dumps(post_body)
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys/' + str(ssh_key_id) + '.json'
    resp = self._s.put(
        url=url,
        data=post_body
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def ssh_keys_view_all(self) -> dict:
    """
    Retrieve all existing SSH Keys.

    :param self:
    :return
        {
            "ResponseType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                "Version":"1.0",
                "Id":"<Id>",
                "Person":
                {
                    "Type":"CO",
                    "Id":"<ID>"
                },
                ,
                "Comment":"<Comment>",
                "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                        "ssh-ed25519"|"ssh-rsa"|"ssh-rsa1"),
                "Skey":"<SshKey>",
                "SshKeyAuthenticatorId":"<Id>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status         Response Body       Description
        200 OK              SshKey Response     SSH Keys returned
        401 Unauthorized                        Authentication required
        500 Other Error                         Unknown error
    """
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def ssh_keys_view_per_coperson(self, coperson_id: int) -> dict:
    """
    Retrieve all existing SSH Keys for the specified CO Person.

    :param self:
    :param coperson_id:
    :return
        {
            "ResponseType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                "Version":"1.0",
                "Id":"<Id>",
                "Person":
                {
                    "Type":"CO",
                    "Id":"<ID>"
                },
                ,
                "Comment":"<Comment>",
                "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                        "ssh-ed25519"|"ssh-rsa"|"ssh-rsa1"),
                "Skey":"<SshKey>",
                "SshKeyAuthenticatorId":"<Id>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status             Response Body       Description
        200 OK                  SshKey Response     SSH Keys returned
        401 Unauthorized                            Authentication required
        404 SSH Key Unknown                         id not found
        500 Other Error                             Unknown error
    """
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys.json'
    params = {'copersonid': str(coperson_id)}
    resp = self._s.get(
        url=url,
        params=params
    )
    no_ssh_keys = {
        'RequestType': 'SshKeys',
        'Version': '1.0',
        'SshKeys': []
    }
    if resp.status_code == 200:
        return json.loads(resp.text)
    if resp.status_code == 204:
        return no_ssh_keys
    else:
        resp.raise_for_status()


def ssh_keys_view_one(self, ssh_key_id: int) -> dict:
    """
    Retrieve an existing SSH Key.

    :param self:
    :param ssh_key_id:
    :return
        {
            "ResponseType":"SshKeys",
            "Version":"1.0",
            "SshKeys":
            [
                {
                "Version":"1.0",
                "Id":"<Id>",
                "Person":
                {
                    "Type":"CO",
                    "Id":"<ID>"
                },
                ,
                "Comment":"<Comment>",
                "Type":("ssh-dss"|"ecdsa-sha2-nistp256"|"ecdsa-sha2-nistp384"|"ecdsa-sha2-nistp521"|
                        "ssh-ed25519"|"ssh-rsa"|"ssh-rsa1"),
                "Skey":"<SshKey>",
                "SshKeyAuthenticatorId":"<Id>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status             Response Body       Description
        200 OK                  SshKey Response     SSH Keys returned
        401 Unauthorized                            Authentication required
        404 SSH Key Unknown                         id not found
        500 Other Error                             Unknown error
    """
    url = self._CO_API_URL + '/ssh_key_authenticator/ssh_keys/' + str(ssh_key_id) + '.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



# comanage_api/_identifiers.py

"""
Identifier API - https://spaces.at.internet2.edu/display/COmanage/Identifier+API

Methods
-------
identifiers_add() -> dict
    ### NOT IMPLEMENTED ###
    Add a new Identifier.
identifiers_assign() -> bool
    ### NOT IMPLEMENTED ###
    Assign Identifiers for a CO Person.
identifiers_delete() -> bool
    ### NOT IMPLEMENTED ###
    Remove an Identifier.
identifiers_edit() -> bool
    ### NOT IMPLEMENTED ###
    Edit an existing Identifier.
identifiers_view_all() -> dict
    Retrieve all existing Identifiers.
identifiers_view_per_entity(entity_type: str, entity_id: int) -> dict
    Retrieve Identifiers attached to a CO Department, Co Group, CO Person, or Org Identity.
identifiers_view_one(identifier_id: int) -> dict
    Retrieve all existing Identifiers.
"""

import json


def identifiers_add(self) -> dict:
    """
    ### NOT IMPLEMENTED ###
    Add a new Identifier.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def identifiers_assign(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Assign Identifiers for a CO Person.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def identifiers_delete(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Remove an Identifier.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def identifiers_edit(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Edit an existing Identifier.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def identifiers_view_all(self) -> dict:
    """
    Retrieve all existing Identifiers.

    :param self:
    :return
        {
          "ResponseType":"Identifiers",
          "Version":"1.0",
          "Identifiers":
          [
            {
              "Version":"1.0",
              "Id":"<ID>",
              "Type":"<Type>",
              "Identifier":"<Identifier>",
              "Login":true|false,
              "Person":{"Type":("CO"|"Dept"|"Group"|"Org"|"Organization"),"ID":"<ID>"},
              "CoProvisioningTargetId":"<CoProvisioningTargetId>",
              "Status":"Active"|"Deleted",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>"
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status         Response Body           Description
        200 OK              Identifier Response     Identifiers returned
        401 Unauthorized                            Authentication required
        500 Other Error                             Unknown error
    """
    url = self._CO_API_URL + '/identifiers.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def identifiers_view_per_entity(self, entity_type: str, entity_id: int) -> dict:
    """
    Retrieve Identifiers attached to a CO Department, Co Group, CO Person, or Org Identity.

    :param self:
    :param entity_type:
    :param entity_id:
    :return
        {
          "ResponseType":"Identifiers",
          "Version":"1.0",
          "Identifiers":
          [
            {
              "Version":"1.0",
              "Id":"<ID>",
              "Type":"<Type>",
              "Identifier":"<Identifier>",
              "Login":true|false,
              "Person":{"Type":("CO"|"Dept"|"Group"|"Org"|"Organization"),"ID":"<ID>"},
              "CoProvisioningTargetId":"<CoProvisioningTargetId>",
              "Status":"Active"|"Deleted",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>"
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      Identifier Response     Identifier returned
        204 CO Department                                   The requested CO Department was found,
            Has No Identifier                                   but has no identifiers attached
        204 CO Group                                        The requested CO Group was found,
            Has No Identifier                                   but has no identifiers attached
        204 CO Person                                       The requested CO Person was found,
            Has No Identifier                                   but has no identifiers attached
        204 Organization                                    The requested Organization was found,
            Has No Identifier                                   but has no identifiers attached
        204 Org Identity                                    The requested Org Identity was found,
            Has No Identifier                                   but has no identifiers attached
        401 Unauthorized                                    Authentication required
        404 CO Department Unknown                           id not found for CO Department
        404 CO Group Unknown                                id not found for CO Group
        404 CO Person Unknown                               id not found for CO Person
        404 Organization Unknown                            id not found for Organization
        404 Org Identity Unknown                            id not found for Org Identity
        500 Other Error                                     Unknown error
    """
    if not entity_type:
        entity_type = 'copersonid'
    else:
        entity_type = str(entity_type).lower()
    if entity_type not in self.ENTITY_OPTIONS:
        raise TypeError("Invalid Fields 'entity_type'")
    url = self._CO_API_URL + '/identifiers.json'
    params = {str(entity_type): str(entity_id)}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def identifiers_view_one(self, identifier_id: int) -> dict:
    """
    Retrieve all existing Identifiers.

    :param self:
    :param identifier_id:
    :return
        {
          "ResponseType":"Identifiers",
          "Version":"1.0",
          "Identifiers":
          [
            {
              "Version":"1.0",
              "Id":"<ID>",
              "Type":"<Type>",
              "Identifier":"<Identifier>",
              "Login":true|false,
              "Person":{"Type":("CO"|"Dept"|"Group"|"Org"|"Organization"),"ID":"<ID>"},
              "CoProvisioningTargetId":"<CoProvisioningTargetId>",
              "Status":"Active"|"Deleted",
              "Created":"<CreateTime>",
              "Modified":"<ModTime>"
            },
            {...}
          ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      Identifier Response     Identifiers returned
        401 Unauthorized                                    Authentication required
        404 Identifier Unknown                              id not found
        500 Other Error                                     Unknown error
    """
    url = self._CO_API_URL + '/identifiers/' + str(identifier_id) + '.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



# comanage_api/_names.py

"""
Name API - https://spaces.at.internet2.edu/display/COmanage/Name+API

Methods
-------
names_add() -> dict
    ### NOT IMPLEMENTED ###
    Add a new Name.
names_delete() -> bool
    ### NOT IMPLEMENTED ###
    Remove a Name.
names_edit() -> bool
    ### NOT IMPLEMENTED ###
    Edit an existing Name.
names_view_all() -> dict
    Retrieve all existing Names.
names_view_per_person(person_type: str, person_id: int) -> dict
    Retrieve Names attached to a CO Person or Org Identity.
names_view_one(name_id: int) -> dict
    Retrieve Names attached to a CO Person or Org Identity.
"""

import json


def names_add(self) -> dict:
    """
    ### NOT IMPLEMENTED ###
    Add a new Name.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def names_delete(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Remove a Name.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def names_edit(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Edit an existing Name.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def names_view_all(self) -> dict:
    """
    Retrieve all existing Names.

    :param self:
    :return
        {
            "ResponseType":"Names",
            "Version":"1.0",
            "Names":
            [
                {
                    "Version":"1.0",
                    "Id":"<ID>",
                    "Honorific":"<Honorific>",
                    "Given":"<Given>",
                    "Middle":"<Middle>",
                    "Family":"<Family>",
                    "Suffix":"<Suffix>",
                    "Type":"<Type>",
                    "Language":"<Language>",
                    "PrimaryName":true|false,
                    "Person":
                    {
                        "Type":("CO"|"Org"),
                        "Id":"<ID>"
                    }
                    "Created":"<CreateTime>",
                    "Modified":"<ModTime>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status         Response Body           Description
        200 OK              Name Response           Name returned
        401 Unauthorized                            Authentication required
        500 Other Error                             Unknown error
    """
    url = self._CO_API_URL + '/names.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def names_view_per_person(self, person_type: str, person_id: int) -> dict:
    """
    Retrieve Names attached to a CO Person or Org Identity.

    :param self:
    :param person_type:
    :param person_id:
    :return
        {
            "ResponseType":"Names",
            "Version":"1.0",
            "Names":
            [
                {
                    "Version":"1.0",
                    "Id":"<ID>",
                    "Honorific":"<Honorific>",
                    "Given":"<Given>",
                    "Middle":"<Middle>",
                    "Family":"<Family>",
                    "Suffix":"<Suffix>",
                    "Type":"<Type>",
                    "Language":"<Language>",
                    "PrimaryName":true|false,
                    "Person":
                    {
                        "Type":("CO"|"Org"),
                        "Id":"<ID>"
                    }
                    "Created":"<CreateTime>",
                    "Modified":"<ModTime>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      Name Response           Name returned
        401 Unauthorized                                    Authentication required
        404 CO Person Unknown                               id not found for CO Person
        404 Org Identity Unknown                            id not found for Org Identity
        500 Other Error                                     Unknown error
    """
    if not person_type:
        person_type = 'copersonid'
    else:
        person_type = str(person_type).lower()
    if person_type not in self.PERSON_OPTIONS:
        raise TypeError("Invalid Fields 'person_type'")
    url = self._CO_API_URL + '/names.json'
    params = {str(person_type): str(person_id)}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def names_view_one(self, name_id: int) -> dict:
    """
    Retrieve Names attached to a CO Person or Org Identity.

    :param self:
    :param name_id:
    :return
        {
            "ResponseType":"Names",
            "Version":"1.0",
            "Names":
            [
                {
                    "Version":"1.0",
                    "Id":"<ID>",
                    "Honorific":"<Honorific>",
                    "Given":"<Given>",
                    "Middle":"<Middle>",
                    "Family":"<Family>",
                    "Suffix":"<Suffix>",
                    "Type":"<Type>",
                    "Language":"<Language>",
                    "PrimaryName":true|false,
                    "Person":
                    {
                        "Type":("CO"|"Org"),
                        "Id":"<ID>"
                    }
                    "Created":"<CreateTime>",
                    "Modified":"<ModTime>"
                },
                {...}
            ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      Name Response           Name returned
        401 Unauthorized                                    Authentication required
        404 Name Unknown                                    id not found
        500 Other Error                                     Unknown error
    """
    url = self._CO_API_URL + '/names/' + str(name_id) + '.json'
    resp = self._s.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



# comanage_api/_copeople.py

"""
CoPerson API - https://spaces.at.internet2.edu/display/COmanage/CoPerson+API

Methods
-------
copeople_add() -> dict
    ### NOT IMPLEMENTED ###
    Add a new CO Person. A person must have an OrgIdentity before they can be added to a CO.
    Note that linking to an OrgIdentity and invitations are separate operations.
copeople_delete() -> bool
    ### NOT IMPLEMENTED ###
    Remove a CO Person. This method will also delete related data, such as CoPersonRoles, EmailAddresses,
    and Identifiers. A person must be removed from any COs (CoPerson records must be deleted)
    before the OrgIdentity record can be removed.
copeople_edit() -> bool
    ### NOT IMPLEMENTED ###
    Edit an existing CO Person.
copeople_find() -> dict
    ### NOT IMPLEMENTED ###
    Search for existing CO Person records.
    When too many records are found, a message may be returned rather than specific records.
copeople_match(given: str = None, family: str = None, mail: str = None, distinct_by_id: bool = True) -> dict
    Attempt to match existing CO Person records.
    Note that matching is not performed on search criteria of less than 3 characters,
    or for email addresses that are not syntactically valid.
copeople_view_all() -> dict
    Retrieve all existing CO People for the specified CO.
copeople_view_per_identifier(identifier: str, distinct_by_id: bool = True) -> dict
    Retrieve all existing CO People attached to the specified identifier.
    Note the specified identifier must be attached to a CO Person, not an Org Identity.
copeople_view_one(coperson_id: int) -> dict
    Retrieve an existing CO Person.
"""

import json


def copeople_add(self) -> dict:
    """
    ### NOT IMPLEMENTED ###
    Add a new CO Person. A person must have an OrgIdentity before they can be added to a CO.
    Note that linking to an OrgIdentity and invitations are separate operations.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 201:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copeople_delete(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Remove a CO Person. This method will also delete related data, such as CoPersonRoles, EmailAddresses,
    and Identifiers. A person must be removed from any COs (CoPerson records must be deleted)
    before the OrgIdentity record can be removed.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def copeople_edit(self) -> bool:
    """
    ### NOT IMPLEMENTED ###
    Edit an existing CO Person.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return True
    else:
        resp.raise_for_status()


def copeople_find(self) -> dict:
    """
    ### NOT IMPLEMENTED ###
    Search for existing CO Person records.
    When too many records are found, a message may be returned rather than specific records.

    :param self:
    :return
        501 Server Error: Not Implemented for url: mock://not_implemented_501.local:
    """
    url = self._MOCK_501_URL
    resp = self._mock_session.get(
        url=url
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copeople_match(self, given: str = None, family: str = None, mail: str = None, distinct_by_id: bool = True) -> dict:
    """
    Attempt to match existing CO Person records.
    Note that matching is not performed on search criteria of less than 3 characters,
    or for email addresses that are not syntactically valid.

    :param self:
    :param given:
    :param family:
    :param mail:
    :param distinct_by_id:
    :return
        {
          "RequestType":"CoPeople",
          "Version":"1.0",
          "CoPeople":
          [
            {
              "Version":"1.0",
              "CoId":"<CoId>",
              "Timezone":"<Timezone>",
              "DateOfBirth":"<DateOfBirth>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Locked"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended")
            }
          ]
        }:

    Response Format
        HTTP Status             Response Body           Description
        200 OK                  CoPerson Response       CoPerson returned (zero or more matches may be returned)
        401 Unauthorized                                Authentication required
        404 CO Unknown                                  id not found - Not currently implemented
                                                            -- unknown CO will return an empty set
        500 Other Error                                 Unknown error
    """
    url = self._CO_API_URL + '/co_people.json'
    params = {'coid': self._CO_API_ORG_ID}
    if given:
        params.update({'given': given})
    if family:
        params.update({'family': family})
    if mail:
        params.update({'mail': mail})
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        if distinct_by_id:
            resp_dict = json.loads(resp.text)
            distinct_copeople = list({v['Id']: v for v in resp_dict.get('CoPeople')}.values())
            resp_dict['CoPeople'] = distinct_copeople
            return resp_dict
        else:
            return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copeople_view_all(self) -> dict:
    """
    Retrieve all existing CO People for the specified CO.

    :param self:
    :return
        {
          "RequestType":"CoPeople",
          "Version":"1.0",
          "CoPeople":
          [
            {
              "Version":"1.0",
              "CoId":"<CoId>",
              "Timezone":"<Timezone>",
              "DateOfBirth":"<DateOfBirth>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Locked"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended")
            }
          ]
        }:

    Response Format
        HTTP Status             Response Body           Description
        200 OK                  CoPerson Response       CoPerson returned (zero or more matches may be returned)
        401 Unauthorized                                Authentication required
        404 CO Unknown                                  id not found - Not currently implemented
                                                            -- unknown CO will return an empty set
        500 Other Error                                 Unknown error
    """
    url = self._CO_API_URL + '/co_people.json'
    params = {'coid': self._CO_API_ORG_ID}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copeople_view_per_identifier(self, identifier: str, distinct_by_id: bool = True) -> dict:
    """
    Retrieve all existing CO People attached to the specified identifier.
    Note the specified identifier must be attached to a CO Person, not an Org Identity.

    :param self:
    :param identifier:
    :param distinct_by_id:
    :return
        {
          "RequestType":"CoPeople",
          "Version":"1.0",
          "CoPeople":
          [
            {
              "Version":"1.0",
              "CoId":"<CoId>",
              "Timezone":"<Timezone>",
              "DateOfBirth":"<DateOfBirth>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Locked"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended")
            }
          ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      CoPerson Response       CoPerson returned
        401 Unauthorized                                    Authentication required
        404 CO Unknown                                      id not found
        500 Other Error                                     Unknown error
    """
    url = self._CO_API_URL + '/co_people.json'
    params = {'coid': self._CO_API_ORG_ID, 'search.identifier': identifier}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        if distinct_by_id:
            resp_dict = json.loads(resp.text)
            distinct_copeople = list({v['Id']: v for v in resp_dict.get('CoPeople')}.values())
            resp_dict['CoPeople'] = distinct_copeople
            return resp_dict
        else:
            return json.loads(resp.text)
    else:
        resp.raise_for_status()


def copeople_view_one(self, coperson_id: int) -> dict:
    """
    Retrieve an existing CO Person.

    :param self:
    :param coperson_id:
    :return
        {
          "RequestType":"CoPeople",
          "Version":"1.0",
          "CoPeople":
          [
            {
              "Version":"1.0",
              "CoId":"<CoId>",
              "Timezone":"<Timezone>",
              "DateOfBirth":"<DateOfBirth>",
              "Status":("Active"|"Approved"|"Confirmed"|"Declined"|"Deleted"|"Denied"|"Duplicate"|"Expired"|
                  "GracePeriod"|"Invited"|"Locked"|"Pending"|"PendingApproval"|"PendingConfirmation"|"Suspended")
            }
          ]
        }:

    Response Format
        HTTP Status                 Response Body           Description
        200 OK                      CoPerson Response       CoPerson returned
        401 Unauthorized                                    Authentication required
        404 copeople Unknown                                id not found
        500 Other Error                                     Unknown error
    """
    url = self._CO_API_URL + '/co_people/' + str(coperson_id) + '.json'
    params = {'coid': self._CO_API_ORG_ID}
    resp = self._s.get(
        url=url,
        params=params
    )
    if resp.status_code == 200:
        return json.loads(resp.text)
    else:
        resp.raise_for_status()



import requests_mock
from requests import Session

from ._copeople import copeople_add, copeople_delete, copeople_edit, copeople_find, copeople_match, \
    copeople_view_all, copeople_view_per_identifier, copeople_view_one
from ._copersonroles import copersonroles_add, copersonroles_delete, copersonroles_edit, copersonroles_view_all, \
    copersonroles_view_per_coperson, copersonroles_view_per_cou, copersonroles_view_one
from ._cous import cous_add, cous_delete, cous_edit, cous_view_all, cous_view_per_co, cous_view_one
from ._identifiers import identifiers_add, identifiers_assign, identifiers_delete, identifiers_edit, \
    identifiers_view_all, identifiers_view_per_entity, identifiers_view_one
from ._names import names_add, names_delete, names_edit, names_view_all, names_view_per_person, names_view_one
from ._sshkeys import ssh_keys_add, ssh_keys_delete, ssh_keys_edit, ssh_keys_view_all, ssh_keys_view_per_coperson, \
    ssh_keys_view_one

# fabric-comanage-api version
__VERSION__ = "0.1.2"


class ComanageApi(object):
    """
    fabric-comanage-api:

    Provide a limited Python 3 client implementation (wrapper) for
    COmanage REST API v1: https://spaces.at.internet2.edu/display/COmanage/REST+API+v1


    Attributes
    ----------
    co_api_url: str
        COmanage registry URL (required)
    co_api_user: str
        COmanage API username (required)
    co_api_pass: str
        COmanage API password (required)
    co_api_org_id: int
        COmanage Org ID (required)
    co_api_org_name: str
        COmanage Org Name (required)
    co_ssh_key_authenticator_id: int = None
        SSH Authenticator Plugin ID (optional)
    """

    def __init__(self, co_api_url: str, co_api_user: str, co_api_pass: str, co_api_org_id: int,
                 co_api_org_name: str, co_ssh_key_authenticator_id: int = None):
        # COmanage API user and pass
        self._CO_API_USER = str(co_api_user)
        self._CO_API_PASS = str(co_api_pass)
        # COmanage CO information
        self._CO_API_ORG_NAME = str(co_api_org_name)
        self._CO_API_ORG_ID = int(co_api_org_id)
        # COmanage Registry URL
        if str(co_api_url).endswith('/'):
            self._CO_API_URL = str(co_api_url)[:-1]
        else:
            self._CO_API_URL = str(co_api_url)
        # COmanage SshKeyAuthenticatorId
        if co_ssh_key_authenticator_id:
            self._CO_SSH_KEY_AUTHENTICATOR_ID = int(co_ssh_key_authenticator_id)
        else:
            self._CO_SSH_KEY_AUTHENTICATOR_ID = 0
        # Status Type options
        self.STATUS_OPTIONS = ['Active', 'Approved', 'Confirmed', 'Declined', 'Deleted', 'Denied', 'Duplicate',
                               'Expired',
                               'GracePeriod', 'Invited', 'Pending', 'PendingApproval', 'PendingConfirmation',
                               'Suspended']
        # Affiliation Type options
        self.AFFILIATION_OPTIONS = ['affiliate', 'alum', 'employee', 'faculty', 'member', 'staff', 'student']
        # Entity Type options
        self.ENTITY_OPTIONS = ['codeptid', 'cogroupid', 'copersonid', 'organizationid', 'orgidentityid']
        # Person Type options
        self.PERSON_OPTIONS = ['copersonid', 'orgidentityid']
        # SSH Key Type options
        self.SSH_KEY_OPTIONS = ['ssh-dss', 'ecdsa-sha2-nistp256', 'ecdsa-sha2-nistp384', 'ecdsa-sha2-nistp521',
                                'ssh-ed25519', 'ssh-rsa', 'ssh-rsa1']
        # create mock response session
        self._mock_session = Session()
        self._adapter = requests_mock.Adapter()
        self._mock_session.mount('mock://', self._adapter)
        # add mock adapters
        self._MOCK_501_URL = 'mock://not_implemented_501.local'
        self._adapter.register_uri('GET', self._MOCK_501_URL, reason='Not Implemented', status_code=501)
        # create comanage_api session
        self._s = Session()
        self._s.auth = (self._CO_API_USER, self._CO_API_PASS)

    # COperson API
    def copeople_add(self):
        return copeople_add(self)

    def copeople_delete(self):
        return copeople_delete(self)

    def copeople_edit(self):
        return copeople_edit(self)

    def copeople_find(self):
        return copeople_find(self)

    def copeople_match(self, given: str = None, family: str = None, mail: str = None, distinct_by_id: bool = True):
        return copeople_match(self, given=given, family=family, mail=mail, distinct_by_id=distinct_by_id)

    def copeople_view_all(self):
        return copeople_view_all(self)

    def copeople_view_per_identifier(self, identifier: str, distinct_by_id: bool = True):
        return copeople_view_per_identifier(self, identifier=identifier, distinct_by_id=distinct_by_id)

    def copeople_view_one(self, coperson_id: int):
        return copeople_view_one(self, coperson_id=coperson_id)

    # COPersonRoles API
    def copersonroles_add(self, coperson_id: int, cou_id: int, status: str = None, affiliation: str = None):
        return copersonroles_add(self, coperson_id=coperson_id, cou_id=cou_id, status=status, affiliation=affiliation)

    def copersonroles_delete(self, copersonrole_id: int):
        return copersonroles_delete(self, copersonrole_id=copersonrole_id)

    def copersonroles_edit(self, copersonrole_id: int, coperson_id: int = None, cou_id: int = None, status: str = None,
                           affiliation: str = None):
        return copersonroles_edit(self, copersonrole_id=copersonrole_id, coperson_id=coperson_id, cou_id=cou_id,
                                  status=status, affiliation=affiliation)

    def copersonroles_view_all(self):
        return copersonroles_view_all(self)

    def copersonroles_view_per_coperson(self, coperson_id: int):
        return copersonroles_view_per_coperson(self, coperson_id=coperson_id)

    def copersonroles_view_per_cou(self, cou_id: int):
        return copersonroles_view_per_cou(self, cou_id=cou_id)

    def copersonroles_view_one(self, copersonrole_id: int):
        return copersonroles_view_one(self, copersonrole_id=copersonrole_id)

    # COU API
    def cous_add(self, name: str, description: str, parent_id: int = None):
        return cous_add(self, name=name, description=description, parent_id=parent_id)

    def cous_delete(self, cou_id: int):
        return cous_delete(self, cou_id=cou_id)

    def cous_edit(self, cou_id: int, name: str = None, description: str = None, parent_id: int = None):
        return cous_edit(self, cou_id=cou_id, name=name, description=description, parent_id=parent_id)

    def cous_view_all(self):
        return cous_view_all(self)

    def cous_view_per_co(self):
        return cous_view_per_co(self)

    def cous_view_one(self, cou_id: int):
        return cous_view_one(self, cou_id=cou_id)

    # Indentifier API
    def identifiers_add(self):
        return identifiers_add(self)

    def identifiers_assign(self):
        return identifiers_assign(self)

    def identifiers_delete(self):
        return identifiers_delete(self)

    def identifiers_edit(self):
        return identifiers_edit(self)

    def identifiers_view_all(self):
        return identifiers_view_all(self)

    def identifiers_view_per_entity(self, entity_type: str, entity_id: int):
        return identifiers_view_per_entity(self, entity_type=entity_type, entity_id=entity_id)

    def identifiers_view_one(self, identifier_id: int):
        return identifiers_view_one(self, identifier_id=identifier_id)

    # Name API
    def names_add(self):
        return names_add(self)

    def names_delete(self):
        return names_delete(self)

    def names_edit(self):
        return names_edit(self)

    def names_view_all(self):
        return names_view_all(self)

    def names_view_per_person(self, person_type: str, person_id: int):
        return names_view_per_person(self, person_type=person_type, person_id=person_id)

    def names_view_one(self, name_id: int):
        return names_view_one(self, name_id=name_id)

    # SshKey API
    def ssh_keys_add(self, coperson_id: int, ssh_key: str, key_type: str, comment: str = None,
                     ssh_key_authenticator_id: int = None):
        return ssh_keys_add(self, coperson_id=coperson_id, ssh_key=ssh_key, key_type=key_type, comment=comment,
                            ssh_key_authenticator_id=ssh_key_authenticator_id)

    def ssh_keys_delete(self, ssh_key_id: int):
        return ssh_keys_delete(self, ssh_key_id=ssh_key_id)

    def ssh_keys_edit(self, ssh_key_id: int, coperson_id: int = None, ssh_key: str = None, key_type: str = None,
                      comment: str = None, ssh_key_authenticator_id: int = None):
        return ssh_keys_edit(self, ssh_key_id=ssh_key_id, coperson_id=coperson_id, ssh_key=ssh_key, key_type=key_type,
                             comment=comment, ssh_key_authenticator_id=ssh_key_authenticator_id)

    def ssh_keys_view_all(self):
        return ssh_keys_view_all(self)

    def ssh_keys_view_per_coperson(self, coperson_id: int):
        return ssh_keys_view_per_coperson(self, coperson_id=coperson_id)

    def ssh_keys_view_one(self, ssh_key_id: int):
        return ssh_keys_view_one(self, ssh_key_id=ssh_key_id)
